﻿[CmdletBinding()]
param
(
    [Parameter(Mandatory=$true)]
    [Uri]
    $destination,

    [Parameter(Mandatory=$true)]
    [string]
    $username,

    [Parameter(Mandatory=$true)]
    [Security.SecureString]
    $password,

    [Parameter(Mandatory=$true)]
    [string]
    $hostname,

    [Parameter(Mandatory=$true)]
    [int]
    $timeout
)

Set-StrictMode -Version Latest
$ErrorActionPreference = 'Stop'

# 必要なコマンドを設定する。
Import-Module VMware.VumAutomation
Set-PowerCLIConfiguration -DefaultVIServerMode 'single' -InvalidCertificateAction 'Ignore' -Scope 'Session' -WebOperationTimeoutSeconds $timeout -Confirm:$false > $null

# vCenter Server に接続する。
$credential = New-Object System.Management.Automation.PSCredential "$username", $password
$vi = Connect-VIServer -Server $destination.Host -Port $destination.Port -Credential $credential -Notdefault -Force

# 対象の ESXi を取得する。
$esxi = Get-VMHost -Name "$hostname" -Server $vi

# コンプライアンスを確認する。
Test-Compliance -Entity $esxi -Server $vi > $null

# コンプライアンス状態を取得する。
$results = Get-Compliance -Entity $esxi -Server $vi -ComplianceStatus 'NotCompliant' -Detailed

# 非準拠のパッチを返却する。
$patches = @()

foreach($compliance in @($results))
{
    switch -Regex ($compliance.GetType().Name)
    {
        "^BaselineCompliance(Impl)?$"
        {
            $baseline = $compliance.Baseline

            $obj = New-Object System.Dynamic.ExpandoObject
            $obj.Activates = @('Reboot')
            $obj.Description = $baseline.UpgradeRelease.ESXProduct
            $obj.Name = $baseline.UpgradeRelease.Name
            $obj.Version = $baseline.UpgradeRelease.Build
            $patches += ,$obj
        }
        "^PatchBaselineCompliance(Impl)?$"
        {
            foreach ($patch in @($compliance.NotCompliantPatches))
            {
                $obj = New-Object System.Dynamic.ExpandoObject
                $obj.Activates = @($patch.InstallationImpact)
                $obj.Description = $patch.Name
                $obj.Name = $patch.IdByVendor
                $obj.Version = $patch.ReleaseDate.ToString()
                $patches += ,$obj
            }
        }
        default { }
    }
}

Write-Output $patches
